import * as React from 'react';
import { ITreeItem, ITreeView, ITreeViewDataProvider } from '@workbench-stack/core';
import { useAsync } from 'react-async';
import SortableTree, {
    toggleExpandedForAll,
    changeNodeAtPath,
    map,
} from 'react-sortable-tree';


/**
 * CustomViewPanel.tsx
 *
 * Loads Views of a ViewContainer from the Extensions. It is a Filler for Slot component that
 * is filled based on the Extension Information.
 */


export interface ICustomViewPanel {
    treeView: ITreeView;
}


export const CustomViewPanel: React.FC<ICustomViewPanel> = ({ treeView }) => {
    // need to get data from the dataprovider
    const treeData = useAsync<ITreeItem[] | null>(treeView
        ? treeView.dataProvider.getChildren() : Promise.resolve(null));

    const tree = React.useMemo(() => treeData, [ treeData ]);

    function showCollapseAllAction(): boolean {
        return true;
    }

    function refresh(e) {
        treeView.refresh();
    }

    function setVisibility(visible: boolean) {
        setVisibility(visible);
    }



    function layout(height: number) {
        layout(height);
    }


    function setFocus() {

    }

    function getPrimaryActions() {

    }


    function getSecondaryActions() {

    }


    function handleChangeTree(items: ITreeItem[]) {

    }

    return (
        <>
            <SortableTree
                className=""
                theme="default"
                scrollToIndex=""
                canDrop={false}
                treeData={tree}
                onMoveNode={null}
                id="customTreeView"
                onChange={handleChangeTree}
            />
        </>
    );
};
