
/* tslint:disable */
import * as core from '@workbench-stack/core';
import { injectable, inject, postConstruct } from 'inversify';
import { ApolloClient } from 'apollo-client';
import { ClientTypes as CommonTypes } from '@common-stack/client-core';
import { InMemoryCache } from 'apollo-cache-inmemory';


/**
 * Final Configuraiton which includes the merged data. 
 *
 * @export
 * @class ConfigurationService
 * @implements {IClientConfigurationService}
 */
@injectable()
export class ConfigurationService implements core.IBrowserConfigurationService {


    constructor(
        @inject(core.ClientTypes.ILifecycleService)
        private lifecycleService: core.ILifecycleService,
        @inject(CommonTypes.ApolloClient)
        private client: ApolloClient<any>,
        @inject(CommonTypes.InMemoryCache)
        private cache: InMemoryCache,
        @inject(CommonTypes.UtilityClass)
        private utility,
    ) {


    }

    @postConstruct()
    public async getConfiguration() {
        this.lifecycleService.when(core.LifecyclePhase.Ready).then(() =>  {
            this.client.query<core.IFinalConfiguration_WSQuery>({
                query: core.FinalConfiguration_WSDocument,
            }).then(data => console.log('---Configuration Loaded'));
        });
    }


    get configuration() {
        const id = this.utility.getCacheKey({ __typename: core.CacheTypenames.Configuration });
        const result = this.client.readFragment<core.IFinalConfigurationFragment>({
            fragment: core.FinalConfigurationFragmentDoc,
            fragmentName: 'FinalConfiguration',
            id,
        });


        return result;
    }

    getValue<T>(): T;
    getValue<T>(section: string): T;
    getValue<T>(overrides: core.IConfigurationOverrides): T;
    getValue<T>(section: string, overrides: core.IConfigurationOverrides): T;
    getValue(arg1?: any, arg2?: any): any {
        const section = typeof arg1 === 'string' ? arg1 : undefined;
        const overrides = core.isConfigurationOverrides(arg1) ? arg1 : core.isConfigurationOverrides(arg2) ? arg2 : {};
        // return this.configuration.getValue(section, overrides, undefined);
        return core.getConfigurationValue(this.configuration, section, overrides);
    }

    updateValue(key: string, value: any): Promise<void>;
    updateValue(key: string, value: any, overrides: core.IConfigurationOverrides): Promise<void>;
    updateValue(key: string, value: any, target: core.ConfigurationTarget): Promise<void>;
    updateValue(key: string, value: any, overrides: core.IConfigurationOverrides, target: core.ConfigurationTarget): Promise<void>;
    updateValue(key: string, value: any, arg3?: any, arg4?: any): Promise<void> {
        return Promise.reject(new Error('not supported'));
    }

    // inspect<T>(key: string): {
    //     default: T,
    //     user: T,
    //     workspace?: T,
    //     workspaceFolder?: T
    //     value: T
    // } {
    //     return this.configuration.inspect<T>(key, {}, undefined);
    // }

    // keys(): {
    //     default: string[];
    //     user: string[];
    //     workspace: string[];
    //     workspaceFolder: string[];
    // } {
    //     return this.configuration.keys(undefined);
    // }

    reloadConfiguration(folder?: core.IWorkspaceFolder): Promise<void> {
        return Promise.resolve(undefined);
    }


    // private onDidDefaultConfigurationChange(keys: string[]): void {
    //     this._configuration.updateDefaultConfiguration(new DefaultConfigurationModel());
    //     this.trigger(keys, ConfigurationTarget.DEFAULT);
    // }

    // private trigger(keys: string[], source: ConfigurationTarget): void {
    //     this._onDidChangeConfiguration.fire(new ConfigurationChangeEvent().change(keys).telemetryData(source, this.getTargetConfiguration(source)));
    // }

    // private getTargetConfiguration(target: ConfigurationTarget): any {
    //     switch (target) {
    //         case ConfigurationTarget.DEFAULT:
    //             return this._configuration.defaults.contents;
    //         case ConfigurationTarget.USER:
    //             return this._configuration.localUserConfiguration.contents;
    //     }
    //     return {};
    // }



}
