import { IWorkspace, IWorkspaceFolder, WorkbenchState, ISingleFolderWorkspaceIdentifier, IWorkspaceIdentifier, isSingleFolderWorkspaceIdentifier,
    IClientWorkspaceContextService as IWorkspaceContextService,
} from '@workbench-stack/core';
import * as core from '@workbench-stack/core';
import { isEqualOrParent, isEqual } from '@vscode/monaco-editor/esm/vs/base/common/resources';
import { URI } from '@vscode/monaco-editor/esm/vs/base/common/uri';
import { injectable, inject, postConstruct } from 'inversify';
import { ApolloClient } from 'apollo-client';
import { ClientTypes as CommonTypes } from '@common-stack/client-core';
import { InMemoryCache } from 'apollo-cache-inmemory';

@injectable()
export class WorkspaceContextService implements IWorkspaceContextService {

    private workspace: IWorkspace;

    constructor(
        @inject(core.ClientTypes.ILifecycleService)
        private lifecycleService: core.ILifecycleService,
        @inject(CommonTypes.ApolloClient)
        private client: ApolloClient<any>,
        @inject(CommonTypes.InMemoryCache)
        private cache: InMemoryCache,
        @inject(CommonTypes.UtilityClass)
        private utility,
    ) {
    }
    // constructor() {
    //     // this.workspace = new Workspace(workspaceResource.toString(), [toWorkspaceFolder(workspaceResource)]);
    // }

    @postConstruct()
    private async _getWorkspace() {
        this.lifecycleService.when(core.LifecyclePhase.Ready).then(() =>  {
            this.client.query<core.IGetWorkspace_WSQuery>({
                query: core.GetWorkspace_WSDocument,
            }).then(data => this.workspace = data.data.getWorkspace);
        });
    }

    public getFolders(): IWorkspaceFolder[] {
        return this.workspace ? this.workspace.folders : [];
    }

    public getWorkbenchState(): WorkbenchState {
        if (this.workspace.configuration) {
            return WorkbenchState.WORKSPACE;
        }

        if (this.workspace.folders.length) {
            return WorkbenchState.FOLDER;
        }

        return WorkbenchState.EMPTY;
    }

    public getCompleteWorkspace(): Promise<IWorkspace> {
        return Promise.resolve(this.getWorkspace());
    }

    public getWorkspace(): IWorkspace {
        return this.workspace;
    }

    public getWorkspaceFolder(resource: URI): IWorkspaceFolder | null {
        // TODO
        // return this.workspace.getFolder(resource);
        return null;
    }

    public isInsideWorkspace(resource: URI): boolean {
        if (resource && this.workspace) {
            return isEqualOrParent(resource, this.workspace.folders[0].uri);
        }

        return false;
    }

    public isCurrentWorkspace(workspaceIdentifier: ISingleFolderWorkspaceIdentifier | IWorkspaceIdentifier): boolean {
        return isSingleFolderWorkspaceIdentifier(workspaceIdentifier) && isEqual(this.workspace.folders[0].uri, workspaceIdentifier);
    }
}
